<?php

namespace Bydedi\Security;

use Bydedi\Security\Driver\DriverInterface;

/**
 * Class Locker
 * @package Dedi\Security
 *
 * Represente les droits d'accès au ressource pour une requete d'un client
 *
 */
class Locker
{

    /**
     * @var Role[] roles du client qui demande la requete
     */
    protected $roles;

    /**
     * @var DriverInterface
     */
    protected $driver;

    /**
     * @param DriverInterface $driver
     */
    public function __construct(DriverInterface $driver)
    {
        $this->driver = $driver;

        $this->load();
    }

    /**
     * Load role and permission
     *
     * @return void
     */
    public function load()
    {
        $this->roles = $this->driver->getRoles();

        foreach ($this->roles as $key => $role) {
            $permissions = $this->driver->getPermissions($role);
            foreach ($permissions as $permission) {
                $role->addPermission($permission);
            }
        }
    }

    /**
     * @param Role[] $roles
     * @param Permission $permission
     * @return bool
     */
    public function can($roles, Permission $permission)
    {
        if (is_array($roles) == false) {
            $roles = array($roles);
        }

        foreach ($this->roles as $role) {
            if (in_array($role->name, $roles) == true && $role->havePermission($permission) == true) {
                return true;
            }
        }

        return false;
    }

    /**
     * @param string $permissionCode
     * @return Permission|false
     */
    public function findPermissionByCode($permissionCode)
    {
        return $this->driver->findPermissionByCode($permissionCode);
    }

    /**
     * @param $roleName
     * @return Role|false
     */
    public function findRoleByName($roleName)
    {
        return $this->driver->findRoleByName($roleName);
    }
}
